package org.eclipse.smila.integration.trendanalysisresults;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.URL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.blackboard.Blackboard;
import org.eclipse.smila.datamodel.Any;
import org.eclipse.smila.datamodel.AnyMap;
import org.eclipse.smila.datamodel.ipc.IpcAnyReader;
import org.eclipse.smila.processing.Pipelet;
import org.eclipse.smila.processing.ProcessingException;
import org.eclipse.smila.processing.parameters.ParameterAccessor;
import org.eclipse.smila.processing.util.ProcessingConstants;
import org.eclipse.smila.processing.util.ResultCollector;

public class PhotosPipelet implements Pipelet {

	private AnyMap _config;

	protected final Log _log = LogFactory.getLog(getClass());

	private static final String INPUT_ATTRIBUTE_NAME_PAGE_NO = "input_attribute_name_pageNo";
	private static final String INPUT_ATTRIBUTE_NAME_PAGE_SIZE = "input_attribute_name_pageSize";
	//private static final String RESULT_ATTRIBUTE_NAME = "result_attribute_name";

	@Override
	public void configure(AnyMap configuration) throws ProcessingException {
		_config = configuration;
	}

	// Process http://83.212.120.113:8080/TrendAnalyzerResults/resources/popularphotos?pageNo=2&pageSize=9
	@Override
	public String[] process(Blackboard blackboard, String[] recordIds)
			throws ProcessingException {

		final ParameterAccessor paramAccessor = new ParameterAccessor(blackboard, _config);
		final ResultCollector resultCollector = new ResultCollector(paramAccessor, _log, ProcessingConstants.DROP_ON_ERROR_DEFAULT);
		
		String restCall = "http://83.212.120.113:8080/TrendAnalyzerResults/resources/popularphotos";

		if (recordIds != null && recordIds.length > 0)
		{
			String id = recordIds[0];

			try 
			{
				paramAccessor.setCurrentRecord(id);

				// get input parameter name from configuration (bpel file)
				final String pageNoAttribute = paramAccessor.getRequiredParameter(INPUT_ATTRIBUTE_NAME_PAGE_NO);
				final String pageSizeAttribute = paramAccessor.getRequiredParameter(INPUT_ATTRIBUTE_NAME_PAGE_SIZE);

				String pageNo = "", pageSize = "";
				
				// get input parameter value from blackboard (posted json)
				if (blackboard.getMetadata(id).containsKey(pageNoAttribute)) 
				{
					pageNo = blackboard.getMetadata(id).getStringValue(pageNoAttribute);
				}
				
				if (blackboard.getMetadata(id).containsKey(pageSizeAttribute)) 
				{
					pageSize = blackboard.getMetadata(id).getStringValue(pageSizeAttribute);
				}
				
				if (pageNo != null && pageNo.length() > 0)
				{
					restCall = restCall + "?pageNo=" + pageNo;
					
					if (pageSize != null && pageSize.length() > 0)
					{
						restCall = restCall + "&pageSize=" + pageSize;
					}
				}
				else if (pageSize != null && pageSize.length() > 0)
				{
					restCall = restCall + "?pageSize=" + pageSize;
				}
				
				// call rest service
				String output = "";
				Any outputValue = null;
				try
                {
					final URL dmurl = new URL(restCall);
					
                    HttpURLConnection httpCon = (HttpURLConnection) dmurl.openConnection();
                    httpCon.setDoOutput(true);
                    httpCon.setRequestMethod("GET");
                    int status = ((HttpURLConnection) httpCon).getResponseCode();
                    if (status != 200)
                    {
                        output = "PhotosPipelet: " + restCall + ": Response code: " + status;
                        System.err.println(output);
                        outputValue = blackboard.getDataFactory().createStringValue(output);
                    }
                    else
                    {
                    	output = convertStreamToString(httpCon.getInputStream());
                    	IpcAnyReader ipcar = new IpcAnyReader();
                    	outputValue = ipcar.readJsonObject(output);
                    }
                }
                catch (IOException e)
                {
                    output = "IOException during accessing popular photos rest interface: " + e;
                    System.err.println(output);
                    outputValue = blackboard.getDataFactory().createStringValue(output);
                }
				

				// create output parameters
//				Value outputValue1 = blackboard.getDataFactory().createStringValue(output);
//				Value outputValue2 = blackboard.getDataFactory().createDoubleValue(1.1);
//				blackboard.getMetadata(id).put("score", outputValue2);
				
				// put results back into the input structure
				blackboard.getMetadata(id).put("result", outputValue);
				resultCollector.addResult(id);
			} 
			catch (final Exception e) 
			{
				resultCollector.addFailedResult(id, e);
			}
		}
		
		return resultCollector.getResultIds();
	}
	
	private static String convertStreamToString(InputStream is) throws IOException {
        //
        // To convert the InputStream to String we use the
        // Reader.read(char[] buffer) method. We iterate until the
        // Reader return -1 which means there's no more data to
        // read. We use the StringWriter class to produce the string.
        //
        if (is != null) 
        {
            Writer writer = new StringWriter();
 
            char[] buffer = new char[1024];
            try 
            {
                Reader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                int n;
                while ((n = reader.read(buffer)) != -1) 
                {
                    writer.write(buffer, 0, n);
                }
            } 
            finally 
            {
                is.close();
            }
            
            return writer.toString();
        } 
        else 
        {       
            return "";
        }
    }

}
